// 26K22WireCut.c
//
// 2015.01.07 - Jurgen G Schmidt    www.jgscraft.com
// 2015.02.11 - Works!
//
// PIC:           18F26K22 - 28-pin
// Compiler:      CCS PCWH 5.033
// Programmer:    PICkit2
// Hardware:      STD LCD with 4 data lines
//                Schmalzhaus.com EasyDriver V4.4 stepper driver provides 5V to CPU and LCD 
//
//                   MCLR/RE3 - 1    28 - RB7/PGD   LED_HB
//               PB_UP    RA0 - 2    27 - RB6/PGC   
//               PB_DN    RA1 - 3    26 - RB5       JGSDBG
//               PB_OK    RA2 - 4    25 - RB4       
//               PB_DEC   RA3 - 5    24 - RB3       
//               PB_INC   RA4 - 6    23 - RB2       
//                        RA5 - 7    22 - RB1       LCD-RS
//                      - Vss - 8    21 - RB0       LCD-Enable
//            M2STP  OSCI/RA7 - 9    20 - Vdd +         
//            M2ENA  OSCO/RA6 - 10   19 - Vss -
//            M2DIR       RC0 - 11   18 - RC7/RX1   LCD-DB4
//            M1STP       RC1 - 12   17 - RC6/TX1   LCD-DB5
//            M1DIR       RC2 - 13   16 - RC5/SDO   LCD-DB6   
//            M1ENA       RC3 - 14   15 - RC4/SDI   LCD-DB7 
//                    
//------------------------------------- HARDWARE -------------------------------
#include <18F26K22.h>
#device ansi
#device PASS_STRINGS=IN_RAM

#FUSES NOWDT          //No Watch Dog Timer
#FUSES NOPUT          //No Power Up Timer
#FUSES NOMCLR         
#FUSES NOBROWNOUT     //No brownout reset

#use delay(internal=64M)
//#use delay(internal=64M,clock_out)

// Serial debug data via SW. INVERT so can connect directly to legacy PC serial port
#use rs232( BAUD=38400, XMIT=PIN_B5, INVERT, STREAM=JGSDBG )

#define LED_HB    PIN_B7    // heartbeat LED

#define LCD_DB4   PIN_C7    // LCD PINS
#define LCD_DB5   PIN_C6 
#define LCD_DB6   PIN_C5 
#define LCD_DB7   PIN_C4 

#define LCD_E     PIN_B0 
#define LCD_RS    PIN_B1 

#define M1ENA     PIN_C3    // Motor 1 controls - wire feeder
#define M1DIR     PIN_C2  
#define M1STP     PIN_C1    

#define M2ENA     PIN_A6    // Motor 2 controls - wire cutter
#define M2DIR     PIN_C0  
#define M2STP     PIN_A7    

#define PB_UP     PIN_A0    // push buttons
#define PB_DN     PIN_A1
#define PB_OK     PIN_A2
#define PB_DEC    PIN_A3
#define PB_INC    PIN_A4

typedef unsigned int8   UINT8;
typedef unsigned int16  UINT16;
typedef int1            BIT;

//-------------------------------------- INT_TIMER4 ---------------------------
// Tick timer - used for heartbeat only, not needed for wirecutter
//
//  24hrs = 1440min = 86,400secs
//  65,536mins = 45days rollover
//
unsigned int16 intMSecs=0;              // milliseconds counter
unsigned int16 intSecs=0;               // seconds counter
//unsigned int16 intOldSecs=0;            // checks for seconds rollover
unsigned int16 intMins=0;               // minutes counter
//unsigned int16 intOldMins=0;            // checks for minutes rollover

#int_timer4
void SecTick()
{
  intMSecs++;
  //------ increment millisecond timers here
  if( intMSecs > 900L )                 // heartbeat LED
    output_high( LED_HB );
  else
    output_low( LED_HB );

  if( intMSecs > 999L ) {
    intMSecs=0;
    intSecs++;
    //------ increment second timers here
  }
   
  if( intSecs > 59 ) {
    intSecs = 0;   
    intMins++;
    //------ increment minute timers here
  }
  //output_toggle( PIN_A7 );    // verify timing with OScope here
}
//-----------------------------------------------------------------------------

#define   MINDB   3           // debounce count required for button press

#include "flex_lcd2x16.c"     // LCD driver - CCS Forum Code Library #24661 (PCM Programmer)

UINT16  intPBUP, intPBDN, intPBOK, intPBINC, intPBDEC; // debounce counters

void Feed( void );            // feed specified length
void Cut( void );             // operate cutter
void Settings( void );        // set length and quantity

//------ EEPROM INT16 MACROS CCS Forum Post # 40941 by Michael Bradley
// a few internal eeprom macro wrappers

#define mReadByte(addr)         read_eeprom(addr)
#define mWriteByte(addr,data)   write_eeprom(addr,data)

// little endian - remember to increment addr by 2 !!!!
#define mReadWord(addr)         make16( read_eeprom(addr + 1), read_eeprom(addr) )
#define mWriteWord(addr, data)  { write_eeprom(addr, make8(data,0) ); write_eeprom(addr + 1, make8(data,1)); }

UINT16  memLength;      // saved to EEPROM: length of pieces
UINT16  memQuantity;    // saved to EEPROM: quantity to measure and cut
UINT16    i,j;          // loop counters
UINT16    k,q;          // quantity counters

//------------------------------------- main() ---------------------------------

void main()
{
  setup_adc_ports( NO_ANALOGS);         // ALWAYS, Always, always do this !!!

  // Timer2,4,6,8,10 has prescalar 1/4/16 and post 1-16
  // 64MHz -> 16MHz -> .0625us per cycle
  // timer rollover = .0625 x 250 (0xfa) = 15.63us per timer click
  // T2_DIV_BY_16 = 15.63us x 16 = 250us per timer click   
  // Postscalar of 4 gives 1mS tick
  //
  setup_timer_4( T4_DIV_BY_16, 0xf8, 4 ); // Tick timer - adjust as needed via scope

  enable_interrupts(INT_TIMER4);        // for tick timer
  enable_interrupts(GLOBAL);
   
  output_high( M1ENA );                 // turn off steppers... 
  output_high( M2ENA );                 // ...don't need to be energized when idle
  
  delay_ms( 500 );                      // allow serial port and LCD to warm up :)
  fprintf(JGSDBG,"\r\n%s %s %s\r\n",__FILENAME__, __DATE__, __TIME__);

  lcd_init();                           // initialize LCD
  
  lcd_putc("\fWireCutter - 2.0");
  delay_ms( 1000 ); 
  
  k=1;  // manual cutting counter

  //------ see if we have eeprom values - unset memory is FFFF
  //(length is in 1/5 inch quantities starting at 2" (memLength = 10)
  memLength = mReadWord( 0 ); 
  if( memLength < 1000 ) {
    memQuantity = mReadWord( 2 );
  } else {
    memLength = 25;   // default to 5"
    memQuantity = 1;
  }

  //----------------------------------- BIG LOOP -------------------------------
  
  //--- show current settings
  printf( lcd_putc, "\fRDY: %lu at %lu.%lu\""memQuantity, memLength/5L, 2 * (memLength%5L),  );

  while( TRUE ){
   
    //--------------------------------------------------------------------------
    //------ Get Button Press ------
    if( input( PB_UP ) == 0 ) intPBUP++; else intPBUP=0; 
    if( input( PB_DN ) == 0 ) intPBDN++; else intPBDN=0; 
    if( input( PB_OK ) == 0 ) intPBOK++; else intPBOK=0; 
    if( input( PB_DEC ) == 0 ) intPBDEC++; else intPBDEC=0; 

    //--------------------------------------------------------------------------
    //------ Process Press ------
    
    //--- center button - enter/exit setup
    //---
    if( intPBOK == MINDB ) {            
      Settings();
    }
    
    //--- top or leftmost button - start program 1 
    //--- Feed and run automatically per settings
    if( intPBUP == MINDB ) {
      for( q=1; q<=memQuantity; q++ ) {
        printf( lcd_putc, "\fRUNNING:  %lu", q); // running count
        Feed();
        Cut();
        if( input( PB_INC ) == 0 ) break; // allow for e-stop - hold down
      }
      printf( lcd_putc, "\fRDY: %lu at %lu.%lu\""memQuantity, memLength/5L, 2 * (memLength%5L),  );
    }

    //--- bottom or 2nd left button - start program 2 
    //--- Single feed
    if( intPBDN == MINDB ) {
      Feed();
      printf( lcd_putc, "\fFeed: %lu ", k );
      k++;
      delay_ms( 100 );
    }

    //--- left or 2nd right button - start program 3
    //---
    if( intPBDEC == MINDB ) {
      printf( lcd_putc, "\fCutting..." );
      Cut();
      delay_ms( 100 );
    }

    delay_ms( 15 ); // slow loop a little to avoid LCD flicker

  }//end of BIG LOOP
  
}//end of main()
//------------------------------------------------------------------------------

//------------------------------------- Feed() ---------------------------------
void Feed( void )
{
  output_low( M1ENA );
  output_high( M1DIR );

  for( j=325L; j>240; j--){     // Speedup for 1 inch
    output_high( M1STP );
    delay_us( 1 );
    output_low( M1STP );
    delay_us( j*5 );
  }

  for( j=0; j<memLength-10; j++) {
    for( i=0; i<17; i++ ) {     // 1/5 inch    
      output_high( M1STP );
      delay_us( 1 );
      output_low( M1STP );
      delay_us( 1200 );
    }
  }
  
  for( j=240; j<325L; j++){     // slowdown for 1 inch
    output_high( M1STP );
    delay_us( 1 );
    output_low( M1STP );
    delay_us( j*5 );
  }
  output_high( M1ENA );
}
//------------------------------------- Cut() ----------------------------------
void Cut( void )
{
  output_low( M2ENA );        // energize stepper
  output_low( M2DIR );        // set direction of rotation
  for( i=0; i<200; i++ ) {    // one full revolution at full step
    output_high( M2STP );     // send step pulse
    delay_us( 1 );            // length of step pulse
    output_low( M2STP );      // end of step pulse
    delay_us( 1500 );         // allow time for motor to move
  }
  output_high( M2ENA );       // de-energize motor since we don't need to 
                              // actively hold the position.
}                              
//------------------------------------- Settings() -----------------------------
void Settings( void )
{
  UINT8 memToSet = 0;
  #define LEN   0
  #define QUAN  1
  
  while( TRUE )
  {
    //------ Get Button Press ------
    // Ideally, these buttons are physically arranged in a cross with the OK in the center:
    //
    //           UP
    //
    //      DEC  OK  INC
    //
    //           DN
    //
    if( input( PB_UP ) == 0 ) intPBUP++; else intPBUP=0;
    if( input( PB_DN ) == 0 ) intPBDN++; else intPBDN=0; 
    if( input( PB_OK ) == 0 ) intPBOK++; else intPBOK=0; 
    if( input( PB_DEC ) == 0 ) intPBDEC++; else intPBDEC=0; 
    if( input( PB_INC ) == 0 ) intPBINC++; else intPBINC=0; 

    // scroll up/down
    if( intPBUP == MINDB || intPBDN == MINDB ) {
      if( memToSet==LEN ) memToSet=QUAN; else memToSet=LEN; 
    }

    // increment value
    if( intPBINC == MINDB ) {
      if( memToSet==LEN ) memLength++; else memQuantity++; 
    }
    
    // decrement value
    if( intPBDEC == MINDB ) {
      if( memToSet==LEN ) memLength--; else memQuantity--; 
    }

    // set bounds and display    
    switch ( memToSet ) {  
      case LEN:
        if( memLength < 10 ) memLength = 10L;
        printf( lcd_putc, "\fLen: %lu / 5 in", memLength );
        break;
      case QUAN:
        if( memQuantity < 1 ) memLength = 1L;
        printf( lcd_putc, "\fQuantity: %lu", memQuantity );
        break;
    }
   
    // save values and return
    if( intPBOK == MINDB ) {
      mWriteWord( 0, memLength );
      mWriteWord( 2, memQuantity );
      k=0; q=0;  // reset manual and automatic counters
      printf( lcd_putc, "\fRDY: %lu at %lu.%lu\""memQuantity, memLength/5L, 2 * (memLength%5L),  );
      return;
    }
    
    delay_ms( 15 ); // slow loop a little

  }
}
//------------------------------------------------------------------------------
//------ end of 26K22WireCut.c ------
